#include <QHostAddress>
#include <QHostInfo>
#include <QNetworkInterface>
#include <QMessageBox>
#include <QFileDialog>
#include <QTimer>
#include <QSettings>

#include "version.h"
#include "qMediaCenter.h"
#include "deleteButton.h"

QMediaCenter::QMediaCenter(QWidget *parent)
: QDialog(parent)
	,MediaCenterProcess(NULL)
	,TestProcess(NULL)
	,ConfigureProcess(NULL)
{
	setupUi(this);
	setWindowTitle(windowTitle() + " (V" + VERSION + ")");
	if (check_root())
	{
		QMessageBox::critical(this, tr("QMediaCenter"), tr("For security reason, don't use this software as user root"), QMessageBox::Ok);
	}
	check_OS();

	QString s;
	Q_FOREACH(QHostAddress a, QNetworkInterface::allAddresses()) {
		static int i=0;
		if ((a.protocol() == QAbstractSocket::IPv4Protocol) && (a != QHostAddress::LocalHost))
		{
			qDebug() << "address : " << a.toString();
			/*
			 * if more than 1 address, separate them with a coma
			 * To test/debug you can use:
			 * ifconfig eth0:0 192.168.1.35
			 * and to remove this,
			 * ifconfig eth0:0 down
			 */
			if (i++) 
				s.append(", ");
			s.append(a.toString());
		}
	}
	label_IP->setText(s);

	check_httpd();
	check_php();
	check_dir();
	check_autostart();

	// Test if mediacenter is running every second
	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(test()));
	timer->start(1000);
}

void QMediaCenter::check_OS()
{
	QFile file("/etc/lsb-release");

	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		label_OS->setText(tr("NON LSB OS, or Not Linux ??, change the source code !"));
		return;
	}

	QTextStream in(&file);
	QString distrib_id, distrib_rel;

	while (!in.atEnd()) {
		QString line = in.readLine();
		if (line.startsWith("DISTRIB_ID"))
		{
			line.remove(0, sizeof("DISTRIB_ID=") - 1);
			distrib_id = line.simplified();
		}
		if (line.startsWith("DISTRIB_RELEASE="))
		{
			line.remove(0, sizeof("DISTRIB_RELEASE=") - 1);
			distrib_rel = line.simplified();
		}
	}
	label_OS->setText(distrib_id + " " + distrib_rel);
}

bool QMediaCenter::check_root()
{
	QStringList environment = QProcess::systemEnvironment();

	if (environment.contains("USER=root"))
	{
		return true;
	}
	return false;
}

void QMediaCenter::check_httpd()
{
	bool found = false;
	QStringList list;
	QString server;

	list << "/usr/sbin/apache2" << "/usr/sbin/httpd2" << "/usr/sbin/apache" << "/usr/sbin/httpd";

	Q_FOREACH (QString s, list)
	{
		QFile f(s);
		if (f.exists())
		{
			found =true;
			server = s;
			break;
		}
	}
	if (found)
	{
		label_WEB->setText("OK (" + server + ")");
	}
	else
	{
		QPalette palette;
		QBrush brush(QColor(255, 0, 0, 255));
		brush.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Active, QPalette::Text, brush);
		palette.setBrush(QPalette::Inactive, QPalette::Text, brush);
		QBrush brush1(QColor(123, 118, 114, 255));
		brush1.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Disabled, QPalette::Text, brush1);
		label_WEB->setPalette(palette);
		label_WEB->setText(tr("No Web server found"));
	}
}

void QMediaCenter::check_php()
{
	QStringList list_dir;
	QString LibExec;
//	QString PHPVersion;
	QString PHPModule;

	list_dir << "/usr/lib/apache2/modules" << "/usr/lib/apache2" << "/usr/lib/apache" << "/usr/libexec/httpd" << "/usr/libexec/apache2" << "/usr/lib/apache/1.3" << "/usr/local/apache/libexec" << "/usr/lib/httpd/modules";
	Q_FOREACH (QString s, list_dir)
	{
		QDir d;
		if (d.exists(s))
		{
			QFile f;
			if (f.exists(s + "/mod_info.so"))
			{
//				qDebug() << s;
				LibExec = s;
				break;
			}
		}
	}
	if (!LibExec.isEmpty())
	{
		QStringList list_php;
		list_php << "php5" << "php4";
		Q_FOREACH (QString s, list_php)
		{
			QFile f;
			if (f.exists(LibExec + "/lib" + s + ".so"))
			{
//				PHPVersion = s;
				PHPModule = LibExec + "/lib" + s + ".so";
//				qDebug() << s;
				break;
			}
		}
	}
	if (!PHPModule.isEmpty())
	{
		label_PHP->setText("OK (" + PHPModule + ")");
	}
	else
	{
		QPalette palette;
		QBrush brush(QColor(255, 0, 0, 255));
		brush.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Active, QPalette::Text, brush);
		palette.setBrush(QPalette::Inactive, QPalette::Text, brush);
		QBrush brush1(QColor(123, 118, 114, 255));
		brush1.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Disabled, QPalette::Text, brush1);
		label_PHP->setPalette(palette);
		label_PHP->setText(tr("No PHP module found"));
	}
}

void QMediaCenter::check_dir()
{
	QSettings settings;
	QDir dir;
	bool valid_dir = true;

	dir.setPath(settings.value("dir",tr("<Choose directory>")).toString());
	label_dir->setText(dir.path());

	label_dir->setPalette(QApplication::palette());
	if (!dir.exists())
	{
		valid_dir = false;
	}
	else
	{
		{
			QFile f(dir.path()+"/configure");
			if ((!f.exists()) | (!(f.permissions() && QFile::ExeUser)))
				valid_dir = false;
		}
		{
			QFile f(dir.path()+"/mediacenter");
			if ((!f.exists()) | (!(f.permissions() && QFile::ExeUser)))
				valid_dir = false;
		}
	}

	if (valid_dir)
	{
		label_download->hide();
		label_download_address->hide();
		if (MediaCenterProcess != NULL)
		{
			MediaCenterProcess->setWorkingDirectory(label_dir->text());
		}
		readShares();
	}
	else
	{
		label_download->show();
		label_download_address->show();

		QPalette palette;
		QBrush brush(QColor(255, 0, 0, 255));
		brush.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Active, QPalette::Text, brush);
		palette.setBrush(QPalette::Inactive, QPalette::Text, brush);
		QBrush brush1(QColor(123, 118, 114, 255));
		brush1.setStyle(Qt::SolidPattern);
		palette.setBrush(QPalette::Disabled, QPalette::Text, brush1);
		label_dir->setPalette(palette);
	}
}

void QMediaCenter::check_autostart()
{
	QSettings settings;
	bool autostart = settings.value("autostart", false).toBool();
	checkBox_Autostart->setChecked(autostart);
	if (autostart)
	{
		on_ButtonStart_clicked();
	}
}

void  QMediaCenter::on_checkBox_Autostart_stateChanged(int state)
{
	QSettings settings;
	switch(state) {
		case Qt::Checked:
			settings.setValue("autostart", true);
			break;
		case Qt::Unchecked:
			settings.setValue("autostart", false);
	}
}

void QMediaCenter::on_ButtonDir_clicked()
{
    qDebug("clicked");


	QString dir = QFileDialog::getExistingDirectory(this, tr("Select Directory of MediaCenter"),
            label_dir->text());

	if (!dir.isEmpty())
	{
		QSettings settings;
		settings.setValue("dir", dir);
		qDebug() << dir;
		label_dir->setText(dir);
		check_dir();
	}
}

void QMediaCenter::on_ButtonSave_clicked()
{
	QFile SharesConf(label_dir->text() + "/data/shares.conf");
	QFile HttpSharesConf(label_dir->text() + "/data/httpshares.conf");
	QTextStream out1(&SharesConf);
	QTextStream out2(&HttpSharesConf);

	SharesConf.open(QIODevice::WriteOnly|QIODevice::Text);
	HttpSharesConf.open(QIODevice::WriteOnly|QIODevice::Text);

	out1 << "<?\n";
	for (int i  = 0; i < SharingTable->rowCount(); i++)
	{
		QByteArray ba;

		// addShareFolder(1, "Vidéos", "/home/stephane/Vidéos");
		out1 << "addShareFolder(" << (i+1) << ", \"" << SharingTable->item(i,0)->text() << "\", \"" << SharingTable->item(i,1)->text() << "\");\n";

		// Alias /__mp9ctl_share_1/ "/home/stephane/Vidéos/"
		out2 << "Alias /__mp9ctl_share_" << (i+1) << "/ \"" << SharingTable->item(i,1)->text() << "/\"\n";
	}
	out1 << "?>\n";
	SharesConf.close();
	HttpSharesConf.close();
}

void QMediaCenter::on_label_dir_textChanged(const QString & newdir)
{
	QSettings settings;
	settings.setValue("dir", newdir);
	check_dir();
}

void QMediaCenter::on_ButtonStart_clicked()
{
	if (MediaCenterProcess == NULL)
	{
		MediaCenterProcess = new QProcess();
		MediaCenterProcess->setWorkingDirectory(label_dir->text());
		connect(MediaCenterProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(On_MediaCenter_Output()));
	}
	MediaCenterProcess->start(label_dir->text() + "/mediacenter", QStringList() << "start");
	qDebug("start clicked");
}

void QMediaCenter::on_ButtonStop_clicked()
{
	if (MediaCenterProcess == NULL)
	{

		MediaCenterProcess = new QProcess();
		MediaCenterProcess->setWorkingDirectory(label_dir->text());
		connect(MediaCenterProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(On_MediaCenter_Output()));
	}
	MediaCenterProcess->start(label_dir->text() + "/mediacenter", QStringList() << "stop");
	qDebug("stop clicked");
}

void QMediaCenter::on_ButtonAdd_clicked()
{
	Ui::addShareDialog ui;
	QDialog *dialog = new QDialog;

	ui.setupUi(dialog);
	if (dialog->exec() == QDialog::Accepted)
	{
		addShare(ui.lineEdit_name->text(), ui.lineEdit_dir->text());
	}
}

void QMediaCenter::on_ButtonAuto_clicked()
{
	QDir q;
	if (q.exists(QDir::homePath()))
		addShare(tr("Home"),QDir::homePath());

	if (q.exists(QDir::homePath() + "/.aMule/Incoming"))
		addShare("aMule",QDir::homePath() + "/.aMule/Incoming");

	if (q.exists(QDir::homePath() + "/" + tr("Video")))
		addShare(tr("Video"),QDir::homePath() + "/" + tr("Video"));

	if (q.exists(QDir::homePath() + "/" + tr("Music")))
		addShare(tr("Music"),QDir::homePath() + "/" + tr("Music"));

	if (q.exists(QDir::homePath() + "/" + tr("Images")))
		addShare(tr("Images"),QDir::homePath() + "/" + tr("Images"));

	if (q.exists("/media"))
		addShare(tr("Media"),"/media");
}

void QMediaCenter::On_MediaCenter_Output()
{
	qDebug() << MediaCenterProcess->readAllStandardOutput();
}

void QMediaCenter::test()
{
	if (TestProcess == NULL)
	{
		TestProcess = new QProcess();
		TestProcess->setWorkingDirectory(label_dir->text());
		connect(TestProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(On_Test_Output()));
	}
	TestProcess->start(label_dir->text() + "/mediacenter", QStringList() << "status");
}

void QMediaCenter::On_Test_Output()
{
	QString r;
	r = TestProcess->readAllStandardOutput();
//	qDebug() << r;
	if (r.contains("stopped"))
	{
		label_status->setText(tr("stopped"));
	}
	else if (r.contains("running"))
	{
		label_status->setText(tr("running"));
	}
	else
	{
		label_status->setText(tr("unknown !"));
	}
}

void QMediaCenter::readShares()
{
	QFile file(label_dir->text() + "/data/shares.conf");

	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		qDebug("no shares.conf file");
		return;
	}

	QTextStream in(&file);
	QString distrib_id, distrib_rel;

	while (!in.atEnd()) {
		QString line = in.readLine();
		if (line.startsWith("addShareFolder("))
		{
			QStringList list = line.split("\"");

			addShare(list.at(1), list.at(3));

			#if 0
			qDebug() << list.at(1);
			qDebug() << list.at(3);
			#endif
		}
	}
}

void QMediaCenter::addShare(const QString & ShareName, const QString & ShareDir)
{
	int i;

	for (i  = 0; i < SharingTable->rowCount(); i++)
	{
		if (SharingTable->item(i,1)->text() == ShareDir)
		{
			qDebug() << "Ignoring duplicate entry" << ShareDir;
			return;
		}

	}

	i = SharingTable->rowCount();
	SharingTable->insertRow(i);
	SharingTable->setItem(i, 0, new QTableWidgetItem(ShareName));
	SharingTable->setItem(i, 1, new QTableWidgetItem(ShareDir));
	DeleteButton *db = new DeleteButton(this);
	SharingTable->setCellWidget(i,2,db);
	connect(db, SIGNAL(delete_line(int)), this, SLOT(On_DeleteLine(int)));
	resetButtons();
	SharingTable->resizeColumnToContents(0);
	SharingTable->resizeColumnToContents(1);
	SharingTable->resizeColumnToContents(2);
}

void QMediaCenter::resetButtons()
{
	for (int i=0; i< SharingTable->rowCount(); i++)
	{
		QWidget *qw;
		DeleteButton *db;
		qw = SharingTable->cellWidget(i,2);
		db = qobject_cast<DeleteButton *>(qw);
		db->setline(i);
	}
}

void QMediaCenter::On_DeleteLine(int line)
{
	QWidget *qw = SharingTable->cellWidget(line, 2);
	DeleteButton *db = qobject_cast<DeleteButton *>(qw);
	db->deleteLater();
	SharingTable->removeRow(line);
	resetButtons();
}
